<?php
namespace SabaiApps\Directories\Component\Display\Element;

use SabaiApps\Directories\Component\Entity;
use SabaiApps\Directories\Component\Display;

class TextElement extends AbstractElement
{
    protected function _displayElementInfo(Entity\Model\Bundle $bundle)
    {
        return array(
            'type' => 'content',
            'label' => _x('Text', 'display element name', 'directories'),
            'description' => __('Adds a block of custom text', 'directories'),
            'default_settings' => array(
                'text' => null,
            ),
            'alignable' => true,
            'positionable' => true,
            'icon' => 'fas fa-bars',
        );
    }
    
    public function displayElementSettingsForm(Entity\Model\Bundle $bundle, array $settings, Display\Model\Display $display, array $parents = [], $tab = null, $isEdit = false, array $submitValues = [])
    {
        return array(
            'text' => array(
                '#title' => __('Text content', 'directories'),
                '#type' => 'textarea',
                '#horizontal' => true,
                '#default_value' => $settings['text'],
                '#required' => true,
                '#description' => sprintf(
                    __('Available tokens: %s', 'directories'),
                    implode(', ', $this->_application->Entity_Tokens($bundle))
                ),
            ),
        );
    }
    
    protected function _getTextFormType()
    {
        if ($this->_application->isComponentLoaded('WordPress')) {
            return 'wp_editor';
        } 
        return 'text';
    }
    
    public function displayElementRender(Entity\Model\Bundle $bundle, array $element, array $display, $var, array $displayOptions, array &$states, $parent = null)
    {
        $text = $this->_application->Entity_Tokens_replace($element['settings']['text'], $var);

        return $this->_application->getPlatform()->htmlize($text);
    }
    
    public function displayElementAdminTitle(Entity\Model\Bundle $bundle, array $element)
    {
        return $this->_application->H($element['settings']['text']);
    }
    
    public function displayElementReadableSettings(Entity\Model\Bundle $bundle, array $settings)
    {
        return [
            'text' => [
                'label' => __('Text content', 'directories'),
                'value' => $settings['text'],
            ],
        ];
    }
}